/* LedTable
 *
 * Written by: Klaas De Craemer, Ing. David Hrbaty  Modified by: Mark Quinn
 * 
 * Menu system for the LED table
 */

#define MINSELECTION  1
#define MAXSELECTION  11
unsigned int curSelection = MINSELECTION;
int begininanimation=1;

#define TEXTSPEED  70 // lower is faster

void mainLoop(void){
  char* curSelectionText;
  int curSelectionTextLength;
  unsigned long prevUpdateTime = 0;
  
  while(true){
    //Show menu system and wait for input
    clearTablePixels();
    switch (curSelection){
      case 1:
        curSelectionText = "1 SD Card";
        curSelectionTextLength = 9;
        break;
      case 2:
        curSelectionText = "2 Tetris";
        curSelectionTextLength = 8;
        break;
      case 3:
        curSelectionText = "3 Snake";
        curSelectionTextLength = 7;      
        break;
      case 4:
        curSelectionText = "4 Pong"; 
        curSelectionTextLength = 6;
        break;  
      case 5:
        curSelectionText = "5 Dice";
        curSelectionTextLength = 6;
        break;    
      case 6:
        curSelectionText = "6 Animation";
        curSelectionTextLength = 11;
        break;    
      case 7:
        curSelectionText = "7 Game of Life";
        curSelectionTextLength = 14;
        break;
      case 8:
        curSelectionText = "8 Rainbow";
        curSelectionTextLength = 9;
        break;     
      case 9:
        curSelectionText = "9 High Scores";
        curSelectionTextLength = 13;
        break;    
      case 10:
        curSelectionText = "10 Message";
        curSelectionTextLength = 10;
        break; 
      case 11:
        curSelectionText = "11 Xmas Tree";
        curSelectionTextLength = 12;
        break;        
    }
    
    boolean selectionChanged = false;
    boolean runSelection = false;
    //Scroll current selection text from right to left;
    for (int x=FIELD_WIDTH; x>-(curSelectionTextLength*8); x--){
      printText(curSelectionText, curSelectionTextLength, x, (FIELD_HEIGHT-8)/2, AQUA);
      //Read buttons
      unsigned long curTime;
      do{
        readInput();
        if (begininanimation == 1){ //this part only runs once at the start of the program and starts it in the animation
          curControl == BTN_START;
          runSelection = true;        
          begininanimation = 0;      
          Serial.println(begininanimation);
        }

        if (curControl != BTN_NONE){
          if (curControl == BTN_LEFT){
            curSelection--;
            selectionChanged = true;
          } else if (curControl == BTN_RIGHT){
            curSelection++;
            selectionChanged = true;
          } else if (curControl == BTN_START){
            runSelection = true;
          }
          
          checkSelectionRange();
        }
        curTime = millis();
      } while (((curTime - prevUpdateTime) < TEXTSPEED) && (curControl == BTN_NONE));//Once enough time  has passed, proceed
      prevUpdateTime = curTime;
      
      if (selectionChanged || runSelection)
        break;
    }
    
    //If we are here, it means a selection was changed or a game started, or user did nothing
    if (selectionChanged){
      //For now, do nothing
    } else if (runSelection){//Start selected game
      switch (curSelection){
        case 1:
          runSDCard();
          break; 
        case 2:
          runTetris();
          break;   
        case 3:
          runSnake();
          break;
        case 4:
          runPong();
          break;    
        case 5:
          runDice();
          break;
        case 6:
          runColorPalette();
          break;
        case 7:
          runGameofLife();         
          break;
        case 8:
          runRainbowPalette();
          break;
        case 9:
          runHighScore();
          break;    
        case 10:
          runMessage();
          break;    
        case 11:
          runTree();
          break;     
      }
    } else {
      //If we are here, no action was taken by the user, so we will move to the next selection automatically
      curSelection++;
      checkSelectionRange();
    }
  }
}

void checkSelectionRange(){
  if (curSelection>MAXSELECTION){
    curSelection = MINSELECTION;
  } else if (curSelection<MINSELECTION){
    curSelection = MAXSELECTION;
  }
}
